<?php
// $Id: nodereference_explorer.menu.inc,v 1.2 2009/06/24 09:27:19 gnindl Exp $
/**
 * @file
 * nodereference_explorer.menu.inc
 * File containing all menu call backs of
 * the module.
 */

/**
 * Menu call back for showing a view item
 * @param $type_name content type
 * @param $field_name label of the CCK field
 * @param $index ascending number of widget (needed for multiple value per widget)
 * @return rendered view item
 */
function nodereference_explorer_preview($type_name, $field_name) {
  if (!empty($_GET['val'])) {
    preg_match('/^(?:\s*|(.*) )?\[\s*nid\s*:\s*(\d+)\s*\]$/', $_GET['val'], $matches);
    if (!empty($matches)) {
      // Explicit [nid:n].
      list(, $title, $nid) = $matches;
    }
  }
  //retrieve the field by name and content type
  $field = content_fields($field_name, $type_name);

  //get the selected rendered view item
  $handler = new nodereference_explorer_views($field);
  $output = $handler->get_item($nid);
  drupal_json(array('status' => TRUE, 'data' => $output));
}

function nodereference_explorer_dialog($type_name, $field_name, $title = '') {
  /* retrieve the field by name
   * @TODO the responsibility of the theme layer varies based on the dialog API
   * so the theme layer needs to accomodate this difference or be abstract
   * so that it doesn't care. right now, it's just a muddle.
   */
  $field = content_fields($field_name, $type_name);
  if (!$field) {
    return '';
  }
  $handler = new nodereference_explorer_views($field);
  $handler->set_selection($title);
  foreach ($handler->get_all_displays() as $display) {
    $tabs[$display->id] = l($display->display_title, $_GET['q'], array('fragment' => $display->id));
//    $filters[$display->id] = $handler->get_exposed_form($display->id);
    $displays[$display->id] = $handler->get_display($display->id);
  }
  nodereference_explorer_add_css('explorer.selection');
  nodereference_explorer_add_css('explorer.tabs');
  jquery_ui_add('ui.tabs');

  //custom action and selection js
  nodereference_explorer_add_js('explorer.selection');
  $output = theme('nodereference_explorer_dialog', $tabs, $filters, $displays);
  switch ($field['widget']['dialog_api']) {
    case 'modalframe':
      $theme = _nodereference_explorer_widget_parse_settings($field, 'theme'); //get theme/css settings
      drupal_add_css(JQUERY_UI_PATH . '/themes/'.$theme['css']); //overall dialog window
      modalframe_child_js();
      return drupal_get_form('nodereference_explorer_dialog_form', $output, $title);
    case NODE_REFERENCE_EXPLORER_MODULE_NAME:
    default:
      $theme = _nodereference_explorer_widget_parse_settings($field, 'theme'); //get theme/css settings
      $dialog = _nodereference_explorer_widget_parse_settings($field, 'dialog');
      //jquery_ui theme
      if (isset($theme['css']) && $theme['css'] != '') {
        drupal_add_css(JQUERY_UI_PATH . '/themes/'.$theme['css']); //overall dialog window
      }
      else { //no custom css specified, either global or module (local) theme
        if (isset($dialog['dialogClass']) && $dialog['dialogClass'] == '') { //no css scope specified, load local module css
         nodereference_explorer_add_css('explorer.dialog');
          // nodereference_explorer_add_css('dialog');
         // nodereference_explorer_add_css('view');
        }
        else { //css scope specified
          //do nothing
          //load global css by specified css scope (done automatically by css scope)
        }
      }
      //add dialog related js files
      $jq_ui_files = array('ui.dialog', 'ui.tabs');

      if ($dialog['draggable']) //can the dialog window be moved
        array_push($jq_ui_files, 'ui.draggable');

      if ($dialog['resizable']) //can the dialog window be resized
        array_push($jq_ui_files, 'ui.resizable');

      if ($dialog['bgiframe']) //patch for IE 6 z-Index problem, i. e. overlay of controls over dialog
        nodereference_explorer_add_js('jquery.bgiframe', true);
      jquery_ui_add($jq_ui_files);

      /* TODO the scripts array contains additional settings and scripts that
       * ideally could inform some process of loading and setting those things.
       * instead, we're calling views_add_js('ajax_view') with the element..
       * $scripts = drupal_add_js(NULL);
       */
      drupal_json(array('status' => TRUE, 'js' => nodereference_explorer_get_js(), 'data' => $output));
  }
}

/*
 *
 * function popups_render_as_json($content) {
 *   $path = $_GET['q']; // Get current path from params.
 *   return drupal_json(array(
 *     'title' => drupal_get_title(),
 *     'messages' => theme('status_messages'),
 *     'path' => $path,
 *     'content' => $content,
 *     'js' => popups_get_js(),
 *     'css' => popups_get_css(),
 *   ));
 * }
 *
 */

function nodereference_explorer_dialog_form($form_state, $dialog, $title) {
  $form['selection'] = array(
    '#type' => 'hidden',
    '#default_value' => $title,
  );
  $form['output'] = array(
    '#value' => $dialog,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Submit',
  );
  return $form;
}

function nodereference_explorer_dialog_form_submit($form, $form_state) {
  modalframe_close_dialog($form_state['values']);
}

/**
 * Get the added JS in a format that is readable by popups.js.
 */
function nodereference_explorer_get_js() {
  $javascript = array_merge_recursive(drupal_add_js(), drupal_add_js(NULL, NULL, 'footer'));
  $output = '';
  $query_string = '?'. drupal_substr(variable_get('css_js_query_string', '0'), 0, 1);

  // For inline Javascript to validate as XHTML, all Javascript containing
  // XHTML needs to be wrapped in CDATA. To make that backwards compatible
  // with HTML 4, we need to comment out the CDATA-tag.
  $embed_prefix = "\n<!--//--><![CDATA[//><!--\n";
  $embed_suffix = "\n//--><!]]>\n";

  foreach ($javascript as $type => $data) {

    if (!$data) continue;

    switch ($type) {
      case 'setting':
        /* Why not just array_merge_recursive($data); */
        $output['setting'] = call_user_func_array('array_merge_recursive', $data);
        break;
      case 'inline':
        foreach ($data as $info) {
          $output['inline'][] = '<script type="text/javascript"' . ($info['defer'] ? ' defer="defer"' : '') . '>' . $embed_prefix . $info['code'] . $embed_suffix . "</script>\n";
        }
        break;
      default:
        foreach ($data as $path => $info) {
          $output[$type][$path] = '<script type="text/javascript"'. ($info['defer'] ? ' defer="defer"' : '') .' src="'. base_path() . $path . $query_string ."\"></script>\n";
        }
    }
  }

  return $output;
}
