<?php
// $Id: nodereference_explorer.widget.inc,v 1.2 2009/06/24 09:27:19 gnindl Exp $
/**
 * @file
 * nodereferene_explorer.widget.inc
 * Widget hooks are placed here. This is the heart class of the module, as it builds on
 * top of the autocomplete widget. It add two buttons (browse and remove). The browse
 * button pops up an interactive dialog. All selected values are written to the autocomplete
 * widget.
 */

/**
 * Implementation of hook_widget_info().
 *
 * Adds the widget type explorer to the
 * nodereference field type
 * @return array of widget types
 */
function nodereference_explorer_widget_info() {
  return array(
    'nodereference_explorer' => array(
      'label' => t('Explorer'),
      'field types' => array('nodereference'),
    ),
  );
}

/**
 * Implementation of hook_widget().
 *
 * This method attaches a single form element to the form. It will be
 * built out and validated in the callback(s) listed in hook_elements.
 * We build it out in the callbacks rather than here in hook_widget so
 * it can be plugged into any module that can provide it with valid
 * $field information.
 *
 * Content module will set the weight, field name and delta values
 * for each form element. This is a change from earlier CCK versions
 * where the widget managed its own multiple values.
 *
 * If there are multiple values for this field, the content module will
 * call this function as many times as needed.
 *
 * @param $form
 *   the entire form array, $form['#node'] holds node information
 * @param $form_state
 *   the form_state, $form_state['values'][$field['field_name']]
 *   holds the field's form values.
 * @param $field
 *   the field array
 * @param $items
 *   array of default values for this field
 * @param $delta
 *   the order of this item in the array of subelements (0, 1, 2, etc)
 *
 * @return
 *   the form item for a single element for this field
 */
// @TODO If the last field on a node form is nodereference_explorer NOT
// multiple, the $form['#cache'] property is set FALSE, which prevents the form
// from being cached. If the form isn't cached, the add more behaviors don't
// work and the fields are deleted from the form. However, if the last field
// on the node form is a multiple nodereference_explorer widget, the add more
// behaviors work fine.
function nodereference_explorer_widget(&$form, &$form_state, $field, $items, $delta = 0) {
  switch ($field['widget']['type']) {
    case NODE_REFERENCE_EXPLORER_MODULE_NAME:
      //core widget (autocomplete)
      $element = array(
        '#type' => NODE_REFERENCE_EXPLORER_MODULE_NAME,
        '#default_value' => isset($items[$delta]) ? $items[$delta] : NULL,
        '#value_callback' => 'nodereference_explorer_widget_value',
      );
      break;
  }
  return $element;
}

/**
 * Proceses a single form element here.
 * For each element an own dialog window is actually rendered including
 * all relevant actions.
 * @param $element
 *   form widget
 * @param $edit
 * @param $form_state
 *   the form_state, $form_state['values'][$field['field_name']] holds the field's form values.
 * @param $form
 *   the entire form array, $form['#node'] holds node information
 * @return $element
 *   widget including javascript
 */
function nodereference_explorer_widget_process($element, $edit, $form_state, $form) {
  $element['#prefix'] = '<div class="nodereference-explorer">';
  $element['#suffix'] = '</div>';
  $setting = array();
  $field = content_fields($element['#field_name'], $element['#type_name']);

  //adds popups references to the widget if present
  if (module_exists('popups_reference') && module_exists('popups'))
    _nodereference_explorer_widget_add_popups_references($element, $field);

  $setting[NODE_REFERENCE_EXPLORER_MODULE_NAME] = array(
    $element['#id'] => array(
      'dialog' => _nodereference_explorer_widget_parse_settings($field, 'dialog'),
      'actions' => _nodereference_explorer_widget_parse_settings($field, 'actions'),
      'path' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_DIALOG .'/'. $element['#type_name'] .'/'. $element['#field_name']),
    ),
  );
  // TODO move content_preview to javascript. this is all about the widget. form element processor
  // should not be responsible for this.
  if ($field['widget']['content_preview'] == '1') {
    $element['preview'] = array(
      '#type' => 'markup',
      '#weight' => 1,
    );

    //add content to preview if available
    $handler = new nodereference_explorer_views($field);
    if ($output = $handler->get_item($element['#default_value']['nid'])) {
      $element['preview']['#value'] = '<div id="'. $element['#id'] .'-preview" class="nodereference-explorer-preview">'. $output .'</div>';
    }
    else { //empty wrapper
      $element['preview']['#value'] = '<div id="'. $element['#id'] .'-preview" class="nodereference-explorer-preview"></div>';
    }

    $setting['nodereference_explorer_preview'] = array(
      $element['#id'] => array(
        'path' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_PREVIEW .'/'. $element['#type_name'] .'/'. $element['#field_name']),
      ),
    );
  }
  drupal_add_js($setting, 'setting');
  return $element;
}

/**
 *
 * Value for a nodereference widget element.
 * Now it just calls the autocomplate value. Any form
 * related manipulation should be done here if necessary.
 * @param $element autocomplete widget
 * @param $edit
 * @return node name and id, eg. name [nid: xx]
 */
function nodereference_explorer_widget_value($element,  $edit = FALSE) {
  $result = nodereference_autocomplete_value($element, $edit);
  return $result;
}


/**
 * Implementation of hook_widget_settings().
 *
 * Configurable content and dialog settings.
 */
function nodereference_explorer_widget_settings($op, $widget) {
  switch ($op) {
    case 'form':
      if ($widget['type'] == NODE_REFERENCE_EXPLORER_MODULE_NAME) {

        $boolean = array(
          t('yes'),
          t('no'),
        );

        //CONTENT SETTINGS
        $form['content'] = array(
          '#type' => 'fieldset',
          '#title' => t('Content'),
          '#collapsible' => TRUE,
          '#collapsed' => FALSE,
          '#description' => t('These settings let you configure the widget related content, e. g. preview.'),
        );

        $form['content']['content_preview'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show preview'),
          '#default_value' => isset($widget['content_preview']) ? $widget['content_preview'] : TRUE,
          '#description' => t('Check if you want to see the node details below the autocomplete field'),
        );

        //DIALOG SETTINGS
        $align = array(
          'center' => t('Center'),
          'left' => t('Left middle'),
          'right' => t('Right middle'),
          'top' => t('Top center'),
          'bottom' => t('Bottom center'),
        );

        //core settings: title, width, height
        $form['dialog'] = array(
          '#type' => 'fieldset',
          '#title' => t('Dialog'),
          '#collapsible' => TRUE,
          '#collapsed' => FALSE,
          '#description' => t('These settings are the configuration options of the jQuery UI dialog window.'),
        );

        $form['dialog']['dialog_title'] = array(
          '#type' => 'textfield',
          '#title' => t('Title'),
          '#default_value' => isset($widget['dialog_title']) ? $widget['dialog_title'] : $widget['label'] . ' ' . t('Explorer'),
          '#required' => TRUE,
          '#description' => t('The label in the title bar.'),
        );

        $form['dialog']['dialog_width@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Width'),
          '#default_value' => isset($widget['dialog_width@int']) ? $widget['dialog_width@int'] : 800,
          '#required' => TRUE,
          '#description' => t('The preferred width of the dialog window.'),
        );

        $form['dialog']['dialog_height@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Height'),
          '#default_value' => isset($widget['dialog_height@int']) ? $widget['dialog_height@int'] : 600,
          '#required' => TRUE,
          '#description' => t('The preferred height of the dialog window.'),
        );

        //INSERTION METHOD
        $dialog_apis = array(
          'nodereference_explorer' => 'Built-in',
        );
        if (module_exists('popups')) {
          $dialog_apis['popups'] = 'Popups API (Ajax Dialogs)';
        }
        if (module_exists('modalframe')) {
          $dialog_apis['modalframe'] = 'Modal Frame API';
        }

        $form['dialog']['dialog_api'] = array(
          '#type' => 'select',
          '#title' => t('Dialog API'),
          '#default_value' => isset($widget['dialog_api']) ? $widget['dialog_api'] : 'nodereference_explorer',
          '#options' => $dialog_apis,
          '#description' => t('Nodereference Explorer supports other Drupal module APIs for dialogs: !popups, !modalframe.',
            array(
              '!popups' => l(theme('placeholder', 'Popups API'), 'http://drupal.org/project/popups', array('html' => TRUE)),
              '!modalframe' => l(theme('placeholder', 'Modal Frame API'), 'http://drupal.org/project/modalframe', array('html' => TRUE)),
            )
          ),
        );

        //alignment of window
        $form['dialog']['align']['dialog_position'] = array(
          '#type' => 'select',
          '#title' => t('Position'),
          '#options' => $align,
          '#default_value' => isset($widget['dialog_position']) ? $widget['dialog_position'] : 'center',
          '#description' => t('The position of the dialog window.'),
        );

        $form['dialog']['theming'] = array(
          '#type' => 'fieldset',
          '#title' => t('Theming'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#description' => t('Leave the following two fields blank if you want to use a global theme (class="drupal-dialog")'),

        );

        global $theme;
        //get all system themes
        $system_themes = system_theme_data();
        //standard themes
        $themes = array(
          t('Module (built-in)') => array('<module>' => t('Default')),
          t('Global (current)') => array('<global>' => $system_themes[$theme]->info['name']),
        );
        //get all jquery ui themes
        $jq_ui_theme_path = JQUERY_UI_PATH .'/themes';
        $jquery_themes =
          file_scan_directory(
            $jq_ui_theme_path, '.*', array('.', '..', 'CVS'), 0, FALSE);

        foreach ($jquery_themes as $jquery_theme)
          if (file_check_directory($jquery_theme->filename))
            $themes['jQuery UI'][$jquery_theme->basename] = ucfirst($jquery_theme->basename);

        $form['dialog']['theming']['theme_css'] = array(
          '#type' => 'select',
          '#options' => $themes,
          '#title' => t('Themes'),
          '#default_value' => isset($widget['theme_css']) ? $widget['theme_css'] : '<module>', //$default_theme_path,
          // '#required' => TRUE,
          '#description' => t('Widget theme composed of module, global and jquery_ui module themes (subpath:<br/>"'. $jq_ui_theme_path .'").'),
        );

        $form['dialog']['theming']['dialog_dialogClass'] = array(
          '#type' => 'textfield',
          '#title' => t('CSS scope'),
          '#default_value' => isset($widget['dialog_dialogClass']) ? $widget['dialog_dialogClass'] : '', //flora',
          '#description' => t('This field allows you to specify a CSS scope to limit your theme to a particular portion of a page. This is helpful when using multiple themes on a page.
            If you don\'t provide a CSS scope, your theme will apply to all UI elements on a page.'),
        );

        $form['dialog']['theming']['dialog_bgiframe@boolean'] = array(
          '#type' => 'select',
          '#title' => t('Background iFrame (IE 6)'),
          '#options' => $boolean,
          '#default_value' => isset($widget['dialog_bgiframe@boolean']) ? $widget['dialog_bgiframe@boolean'] : 0,
          '#description' => t('Loads a background iframe, fixes the IE 6 z-Index issue that background form controls <br/>like a drop-down or select list show on top of the dialog'),
        );


        $form['dialog']['advanced_dimensions'] = array(
          '#type' => 'fieldset',
          '#title' => t('Extra dimensions'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );

        $form['dialog']['advanced_dimensions']['dialog_minWidth@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Minimal width'),
          '#default_value' => isset($widget['dialog_minWidth@int']) ? $widget['dialog_minWidth@int'] : '',
          '#description' => t('The minimal width of the dialog window (leave blank or 0 if no limit).'),
        );

        $form['dialog']['advanced_dimensions']['dialog_minHeight@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Minimal height'),
          '#default_value' => isset($widget['dialog_minHeight@int']) ? $widget['dialog_minHeight@int'] : '',
          '#description' => t('The minimal height of the dialog window (leave blank or 0 if no limit).'),
        );

        $form['dialog']['advanced_dimensions']['dialog_maxWidth@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximal width'),
          '#default_value' => isset($widget['dialog_maxWidth@int']) ? $widget['dialog_maxWidth@int'] : '',
          '#description' => t('The maximal width of the dialog window (leave blank or 0 if no limit).'),
        );

        $form['dialog']['advanced_dimensions']['dialog_maxHeight@int'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximal height'),
          '#default_value' => isset($widget['dialog_maxHeight@int']) ? $widget['dialog_maxHeight@int'] : '',
          '#description' => t('The maximal height of the dialog window (leave blank or 0 if no limit).'),
        );

        $form['dialog']['control'] = array(
          '#type' => 'fieldset',
          '#title' => t('Controls'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );



        $form['dialog']['control']['dialog_draggable@boolean'] = array(
          '#type' => 'select',
          '#title' => t('Draggable'),
          '#options' => $boolean,
          '#default_value' => isset($widget['dialog_draggable@boolean']) ? $widget['dialog_draggable@boolean'] : 0,
          '#description' => t('Specify if the dialog can be dragged by clicking the title bar.'),
        );

        $form['dialog']['control']['dialog_resizable@boolean'] = array(
          '#type' => 'select',
          '#title' => t('Resizable'),
          '#options' => $boolean,
          '#default_value' => isset($widget['dialog_resizable@boolean']) ? $widget['dialog_resizable@boolean'] : 0,
          '#description' => t('Specify if the dialog can be resized.'),
        );

        //close on escape is only supported in jquery ui version > 1.6
       /* $form['dialog_settings']['control']['dialog_settings@closeOnEscape@boolean'] = array(
          '#type' => 'select',
          '#title' => t('Close by escape'),
          '#options' => $boolean,
          '#default_value' => isset($widget['dialog_settings@closeOnEscape@boolean']) ? $widget['dialog_settings@closeOnEscape@boolean'] : 0,
          '#description' => t('Specify if the dialog can be closed when pressing the escape (Esc) button'),
        );*/

        $form['dialog']['effects'] = array(
          '#type' => 'fieldset',
          '#title' => t('Effects'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );


        //effets on dialog open and close
        $effects = array(
          'null'  => t('None'),
          'slide' => t('Slide'),
        );

        $form['dialog']['effects']['dialog_show'] = array(
          '#type' => 'select',
          '#title' => t('Show effect'),
          '#options' => $effects,
          '#default_value' => isset($widget['dialog_show']) ? $widget['dialog_show'] : 'null',
          '#description' => t('Specify the effect when the dialog window opens.'),
        );

        $form['dialog']['effects']['dialog_hide'] = array(
          '#type' => 'select',
          '#title' => t('Close effect'),
          '#options' => $effects,
          '#default_value' => isset($widget['dialog_hide']) ? $widget['dialog_hide'] : 'null',
          '#description' => t('Specify the effect when the dialog window closes.'),
        );

        $form['dialog']['buttons'] = array(
          '#type' => 'fieldset',
          '#title' => t('Button labels'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );

        $form['dialog']['buttons']['actions_ok'] = array(
          '#type' => 'textfield',
          '#title' => t('Confirmation button label'),
          '#default_value' => isset($widget['actions_ok']) ? $widget['actions_ok'] : t('Confirm'),
          '#required' => TRUE,
          '#description' => t('The label of the confirmation button (automatic translation).'),
        );

        $form['dialog']['buttons']['actions_cancel'] = array(
          '#type' => 'textfield',
          '#title' => t('Cancel button label'),
          '#default_value' => isset($widget['actions_cancel']) ? $widget['actions_cancel'] : t('Abort'),
          '#required' => TRUE,
          '#description' => t('The label of the cancel button (automatic translation).'),
        );

        return $form;
      }
      case 'validate':
        nodereference_explorer_include('widget.validate');

        //validate dimensions (mandatory)
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_height@int', 'Height', TRUE);
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_width@int', 'Width', TRUE);

        //validate optional dimensions
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_minHeight@int', 'Minimum height', FALSE);
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_minWidth@int', 'Minimum width', FALSE);
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_maxHeight@int', 'Maximum height', FALSE);
        nodereference_explorer_widget_validate_textfield_number($widget, 'dialog_maxWidth@int', 'Maximum width', FALSE);

        // nodereference_explorer_widget_validate_theme_file($widget, 'theme_css');


        case 'save':
          return array(
          'content_preview',
          'dialog_title',
          'theme_css',
          'dialog_api',
          'dialog_dialogClass',
          'dialog_bgiframe@boolean',
          'dialog_width@int',
          'dialog_height@int',
          'dialog_position',
          'dialog_minWidth@int',
          'dialog_minHeight@int',
          'dialog_maxWidth@int',
          'dialog_maxHeight@int',
          'dialog_draggable@boolean',
          'dialog_resizable@boolean',
          'dialog_show',
          'dialog_hide',
          'actions_ok',
          'actions_cancel',
          );
        }
      }

/**
 * Parsing of widget settings (dialog, content, theme...)
 * Naming convention is {domain}_{key}@type, e. g. dialog_draggable@boolean
 * The typing is need for jQuery in order to interprete the correct data
 * type, f. e. boolean
 * @param $field cck field
 * @param $domain associative key
 * @return array of typed settings, e. g. dialog
 */
function _nodereference_explorer_widget_parse_settings($field, $domain) {
  $settings = array();
  foreach (array_keys($field['widget']) as $widget_key) {
    if (strstr($widget_key, $domain)) {
      //$pos = strpos($widget_key, $domain);
      unset($type);
      if (strstr($widget_key, '@')) {
        $pos_at = strpos($widget_key, '@');
        $type = drupal_substr($widget_key, $pos_at+1);
        $widget_key = drupal_substr($widget_key, 0, $pos_at);
      }
      //$widget_key = substr($widget_key, $pos + strlen($domain));
      $delimit = strpos($widget_key, '_');
      $key = drupal_substr($widget_key, $delimit+1);

      if (isset($type)) { //interprete data type properly
        switch ($type) {
          case 'int':
            $settings[$key] = intval($field['widget'][$widget_key .'@int']);
            break;
          case 'boolean':
            $settings[$key] = $field['widget'][$widget_key .'@boolean'] == 0 ? TRUE : FALSE;
            break;
        }
      }
      else {
        $settings[$key] = $field['widget'][$widget_key];
      }
    }
  }
  return $settings;
}

/**
 * Adds popups references to the widget if enabled
 * Copied and adapted from popups_reference_alter_item()
 */
function _nodereference_explorer_widget_add_popups_references(&$element, $field) {
  if (variable_get('popups_reference_show_add_link_'. $element['#field_name'], TRUE)) {
    $wrapper_id = 'popups-reference-' . _popups_reference_counter();
    $links = _popups_reference_links($field, 'nodereference_explorer', $wrapper_id, 'nodereference_autocomplete');
    if ($links) {
      // Put the nodereference widget and links in an wrapper.
      // Makes it easy to find for Ahah targeting, and popups_reference behavior selecting.
      $element['#prefix'] .= '<div class="'. $wrapper_id .'">';
      $element['#suffix'] .= '<div>Add New: ' . implode(', ', $links) .'</div></div>';
    }
  }
}
