/**
 * The nodereference explorer javascript file.
 * It assigns actions to the widget buttons and draws the
 * dialog. It contains several other fixes now (see code comments).
 * All setting are indexed for multiple values per widget yet.
 */

Drupal.nodereference_explorer.addedJS = [];

/**
 * By clicking the browse button open the dialog
 */
Drupal.nodereference_explorer.actions.browse = function (widget) {
  var id = widget.data('nodereference_explorer');
  var options = Drupal.settings.nodereference_explorer[id].dialog;
  var actions = Drupal.settings.nodereference_explorer[id].actions;
  var path = Drupal.settings.nodereference_explorer[id].path;
  var title = $('.form-autocomplete', widget).val();

  //initialize dialog
  // hard coded defaults
  $.extend(options, {
    modal: true
  });

  var ok = actions.ok;
  var cancel = actions.cancel;
  options.buttons = {};

  //when click ok this happens
  options.buttons[ok] = function() {
    var title = $('#edit-selection', this).val();
    $('.form-autocomplete', widget).val(title).change();
    $(this).dialog('close');
  };

  //when canceling the dialog this happens
  options.buttons[cancel] = function() {
    $(this).dialog('close');
  };

  //when dialog closes this happens
  options.close = function(event) {
    $(this).dialog('destroy');
    $(this).empty();
    Drupal.nodereference_explorer.unmask_title();
  };

  //when the dialog opens this happens
  options.open = function() {
	  Drupal.nodereference_explorer.mask_title();
    $(this).prepend($('<input id="edit-selection" type="hidden" />').val(title));
    Drupal.attachBehaviors(this);
    //tab behaviours
    //also need to pass context value to this tabs(ids) method after it has been cleaned up.
    
  };
  
  //The following events reset the dialog content height to its correct value. jQueryUI 1.6 does not
  //properly commpute these dimenstion when using a button pane.
  options.focus = function() {Drupal.nodereference_explorer.resetDialogContentSize(this, options);};   //on dialog open
  options.resize = function() {Drupal.nodereference_explorer.resetDialogContentSize(this, options);}; //on resizing dialog
  options.resizeStart = function() {Drupal.nodereference_explorer.resetDialogContentSize(this, options);};
  options.resizeStop = function() {Drupal.nodereference_explorer.resetDialogContentSize(this, options);};

  //unhide dialog
  //instantiate the dialog
  $.getJSON(path, function (data, textStatus) {
    var inlines = Drupal.nodereference_explorer.addJS(data.js);
    //add default dialog class specified in the dialog template if no CSS scope has been specified
    if (!options['dialogClass']) 
      options['dialogClass'] = $(data.data).attr('class');
    
    $(data.data).dialog(options);
    
    Drupal.nodereference_explorer.addInlineJS(inlines);
  });
//open the dialog
};

/**
 * Mask the dialog title filter form widgets
 * Helper function for fixing the collision between the filter form's title
 * and the node title
 */
Drupal.nodereference_explorer.mask_title = function () {
	$('#edit-title-wrapper').attr('id', 'edit-title-wrapper-masked');
	$('#edit-title').attr('id', 'edit-title-masked');
};

/**
 * Unmask the dialog title filter form widgets
 * Helper function for fixing the collision between the filter form's title
 * and the node title
 */
Drupal.nodereference_explorer.unmask_title = function () {
	$('#edit-title-wrapper-masked').attr('id', 'edit-title-wrapper');
	$('#edit-title-masked').attr('id', 'edit-title');
};

/**
 * Add additional Javascript to the page.
 */
Drupal.nodereference_explorer.addJS = function(js) {
  // Parse the json info about the new context.
  var scripts = [];
  var inlines = [];
  for (var type in js) {
    if (type != 'setting') {
      for (var file in js[type]) {
        if (type == 'inline') {
          inlines.push($(js[type][file]).text());
        }
        else {
          scripts.push($(js[type][file]).attr('src'));
        }
      }
    }
  }

  // Add new JS settings to the page, needed for #ahah properties to work.
  $.extend(Drupal.settings, js.setting);

  for (var i in scripts) {
    var src = scripts[i];
    if (!$('script[src='+ src + ']').length && !Drupal.nodereference_explorer.addedJS[src]) {
      // Get the script from the server and execute it.
      $.ajax({
        type: 'GET',
        url: src,
        dataType: 'script',
        async : false,
        success: function(script) {
          eval(script);
        }
      });
      // Mark the js as added to the underlying page.
      Drupal.nodereference_explorer.addedJS[src] = true;
    }
  }

  return inlines;
};

/**
 * Execute the jit loaded inline scripts.
 * Q: Do we want to re-excute the ones already in the page?
 *
 * @source
 *   stolen straight from Popups API function Popups.addInlineJS
 * @param inlines
 *   Array of inline scripts.
 */
Drupal.nodereference_explorer.addInlineJS = function(inlines) {
  // Load the inlines into the page.
  for (var n in inlines) {
    // If the script is not already in the page, execute it.
    if (!$('script:not([src]):contains(' + inlines[n] + ')').length) {
      eval(inlines[n]);
    }
  }
};

/**
 * Sets the dialog content to its correct size when using a button pane
 * In jQuery UI 1.6 scrollbars are hidden behind the button pane. This fix
 * actually shortens the height of the actual content by the height of the
 * button pane
 * @param
 *   dialog content
 * @param
 *   options for dialog
 */
Drupal.nodereference_explorer.resetDialogContentSize = function(dialog, options) {
	var container = $(dialog).parent(); //overall container
	var titlebar = $(dialog).siblings('.ui-dialog-titlebar'); //title
	var buttonpane = $(dialog).parent().siblings('.ui-dialog-buttonpane'); //button panel
	
	if (container.height() > 1) { //when container height is already known (all resize and focus events)
	  $container_height = container.height();
	  $buttonpane_height = buttonpane.height();
	}
	else { //when container height is not yet known (opening the dialog)
	  $container_height = options.height;
	  $buttonpane_height = buttonpane.height()/2;
	}
	//console.log($container_height + "," +titlebar.outerHeight() + "," +$buttonpane_height);
	$(dialog).height($container_height - titlebar.outerHeight() - $buttonpane_height);
}
